function c=mldivide(A,b)
%mldivide: Solves the system A\b, at least one of them of class qd, producing another element of class qd

if ~isa(A,'qd') & isa(b,'qd')
    %A simple case; the matrix is double or alike, and the RHS is qd.
    %Split the RHS in four parts, calculate using the standard matlab routines, and
    %reassemble the solution from its parts
    x=A\[getp1(b) getp2(b) getp3(b) getp4(b)];
    [M,N]=size(getp1(b));
    %renormalize just in case
    c=(qd(x(:,1:N))+x(:,(N+1):(2*N))+x(:,(2*N+1):(3*N)))+x(:,(3*N+1):(4*N));
    return
end
%other cases requires special routines. Convert everything to qd
if ~isa(A,'qd')
    A=qd(A);
end
if ~isa(b,'qd')
    b=qd(b);
end
AP1=getp1(A);bP1=getp1(b);
%check for dimensions
if size(AP1,2)~=size(bP1,1)
    error('Wrong size in mldivide:')
end
if (size(AP1,1)==size(AP1,2)) %Square system of equations
    if size(AP1,1)==1
        %scalar case; it is a simple division Ax=b==>x=b/A (change the order of parameters!)
        [out1,out2,out3,out4] = qd_rdivide(getp1(b),getp2(b),getp3(b),getp4(b),getp1(A),getp2(A),getp3(A),getp4(A));
        %reassemble the output matrix from its parts
        c.p1=out1;c.p2=out2;c.p3=out3;c.p4=out4;
        c=class(c,'qd');
    else
        %square matrix; standard system of equations, to be solved by gaussian elimination
        [c,LU,b,shortP]=gausscpp(A,b);
        %build the permutation matrix as a full one
        P=eye(size(LU));P=P(shortP,:);
    end
else
    %either over or undeterminated system of equations.
    %use the QR factorization to solve the problem
    [Q,R]=qr(A,0); %economy size; output matrix R is square
    c=R\(Q'*b);
end

