function c=mpower(A,N)
%mpower:  ^   Matrix power of qd-type base and/or qd-type exponent "A^N"

%     Z = X^y is X to the y power if y is a scalar and X is square. If y
%     is an integer greater than one, the power is computed by repeated
%     squaring. For other values of y the calculation involves
%     eigenvalues and eigenvectors.
%  
%     Z = x^Y is x to the Y power if Y is a square matrix and x is a scalar.
%     Computed using eigenvalues and eigenvectors.
%  
%     Z = X^Y, where both X and Y are matrices, is an error.
%  
%     C = MPOWER(A,B) is called for the syntax 'A ^ B' when A or B is an
%     object.

L_A=double(size(A));L_N=double(size(N));
%check about the type of the base; some cases could be dealt with if it is double
% if isa(A,'qd') 
%     if all(all(getlo(A)==0))
%         %is indeed a double
%         A=gethi(A);
%     end
% end
% %check about the type of the exponent
if isa(N,'qd')
    if all(all(qd(double(N))==N))
        %is indeed a double
        N=double(N);
    end
end

if prod(L_A)==1 
    %the base is scalar
    if prod(L_N)==1
        %scalar^scalar, of any type
        c=power(A,N);
        return
    else
        %exponent is a matrix; it must be square
        if L_N(1)==L_N(2)
            if ~isa(A,'qd') & isa(N,'qd')
                %base of type double, and square matrix expontent of type qd
                %we use a valid trick; A^N=A^(N.p1+N.p2+N.p3+N.p4)=(A^N.p1)*(A^N.p2)*...
                c=qd(A^getp1(N))*qd(A^getp2(N))*qd(A^getp3(N))*qd(A^getp4(N));
                return
            else
                error(['unimplemented yet in mpower.m'])
            end
        else
            error('exponent should be a square matrix if base is scalar')
        end
    end
else
    %the base is a matrix
    if isa(N,'qd') 
        %N of type qd, A matrix of any type
        error(['unimplemented yet in mpower.m'])
    else
        %N is of type double or single
        if prod(L_N)==1 %scalar case
            switch N
                case 1
                    c=A;
                case 2
                    c=A*A;
                case 3
                    c=(A*A)*A;
                case 4
                    c=A*A;
                    c=c*c;
                case 5
                    c=A*A;
                    c=(c*c)*A;
                otherwise
                    error(['unimplemented yet in mpower.m; matrix base qd and non-integer exponent double'])
            end
        else
            %both are matrix, of any type. It is not acceptable, irrespective of the data type
            error(['X^Y could not be matrix^matrix'])
        end
    end
end
