function c=mtimes(a,b)
%mtimes: multiplies two matrices (one of them of class qd), producing another element of class qd "a*b"
%        unless one of the elements is of class intval or gradient. A value of such type is produced in such case
if isa(a,'intval') | isa(b,'intval') | isa(a,'gradient') | isa(b,'gradient')
    %one of the factors is intval, and the other is qd.
    if isa(b,'intval')
        a=intval(a);
        c=a*b;
        return
    end
    if isa(a,'intval')
        b=intval(b);
        c=a*b;
        return
    end
    if isa(b,'gradient')
        c=gradient(a*b.x,a*b.dx);
        return
    end
    if isa(a,'gradient')
        c=gradient(a.x*b,a.dx*b);
        return
    end
end
if ~isa(a,'qd')
    a=qd(a);
end
if ~isa(b,'qd')
    b=qd(b);
end
hi1=getp1(a);hi2=getp1(b);
%check for dimensions
if size(hi1,2)~=size(hi2,1)
    %they have different sizes, which might be an error except if one matrix is a scalar one
    if prod(size(hi1))==1 | prod(size(hi2))==1
        %one is scalar; do not check for dimensions
        if prod(size(hi1))==1 
            %first element is scalar; reshape it to be of the same size of the second term
            a=qd(repmat(getp1(a),size(hi2)),repmat(getp2(a),size(hi2)),repmat(getp3(a),size(hi2)),repmat(getp4(a),size(hi2)));
            [p1,p2,p3,p4] = qd_times(getp1(a),getp2(a),getp3(a),getp4(a),getp1(b),getp2(b),getp3(b),getp4(b));
        else
            %second element is scalar; reshape it to be of the same size of the first term
            b=qd(repmat(getp1(b),size(hi1)),repmat(getp2(b),size(hi1)),repmat(getp3(b),size(hi1)),repmat(getp4(b),size(hi1)));
            [p1,p2,p3,p4] = qd_times(getp1(a),getp2(a),getp3(a),getp4(a),getp1(b),getp2(b),getp3(b),getp4(b));
        end
    else
        %error!
        error(['in mtimes.m; matrices should be have the right size' ])
    end
else
    %invoke the standard function, using the library
    [p1,p2,p3,p4] = qd_mtimes(getp1(a),getp2(a),getp3(a),getp4(a),getp1(b),getp2(b),getp3(b),getp4(b));
end
%renormalization is not needed because it is already done in the C library
s.p1=p1;s.p2=p2;s.p3=p3;s.p4=p4;c=class(s,'qd');
