function c=plus(a,b)
%plus: adds two matrices of class qd, producing another element of class qd "a+b"
%      unless one of the elements is of class intval or gradient. A value of such class is produced in such case
if isa(a,'intval') | isa(b,'intval') | isa(a,'gradient') | isa(b,'gradient')
    %one of the factors is intval or gradient, and the other is qd.
    if isa(b,'intval')
        a=intval(a);
        c=a+b;
        return
    else
        b=intval(b);
        c=a+b;
        return
    end
    if isa(b,'gradient')
        c=gradient(a+b.x,b.dx);
        return
    end
    if isa(a,'gradient')
        c=gradient(a.x+b,a.dx);
        return
    end
end

if ~isa(a,'qd')
    a=qd(a);
end
if ~isa(b,'qd')
    b=qd(b);
end
hi1=getp1(a);hi2=getp2(a);hi3=getp3(a);hi4=getp4(a);
lo1=getp1(b);lo2=getp2(b);lo3=getp3(b);lo4=getp4(b);
%check for dimensions
if size(hi1,1)~=size(lo1,1) | size(hi2,2)~=size(lo2,2)
    %they have different sizes, which might be an error except if one matrix is a scalar one
    if prod(size(hi1))==1 | prod(size(lo1))==1
        %one is scalar; do not check for dimensions
        if prod(size(hi1))==1 
            %first element is scalar; reshape it to be of the same size of the second term
            hi1=repmat(hi1,size(lo1));hi2=repmat(hi2,size(lo1));hi3=repmat(hi3,size(lo1));hi4=repmat(hi4,size(lo1));
        else
            %second element is scalar; reshape it to be of the same size of the first term
            lo1=repmat(lo1,size(hi1));lo2=repmat(lo2,size(hi1));lo3=repmat(lo3,size(hi1));lo4=repmat(lo4,size(hi1));
        end
    else
        %error!
        error(['in plus.m; matrices should be have the right size' ])
    end
end
if isreal(hi1) & isreal(lo1) & isreal(hi2) & isreal(lo2) & isreal(hi3) & isreal(lo3) & isreal(hi4) & isreal(lo4)
    %invoke the standard function, using the library for real variables
    [out1, out2, out3, out4] = qd_plus(hi1,hi2,hi3,hi4,lo1,lo2,lo3,lo4);
    %renormalization is not needed because it is already done in the C library
    s.p1=out1;s.p2=out2;s.p3=out3;s.p4=out4;
    c=class(s,'qd');
else
    %the complex case. Calculate first the real part
    [out1, out2, out3, out4] = qd_plus(real(hi1),real(hi2),real(hi3),real(hi4),...
        real(lo1),real(lo2),real(lo3),real(lo4));
    %renormalization is not needed because it is already done in the C library
    s.p1=out1;s.p2=out2;s.p3=out3;s.p4=out4;
    %now the complex one
    [out1, out2, out3, out4] = qd_plus(imag(hi1),imag(hi2),imag(hi3),imag(hi4),...
        imag(lo1),imag(lo2),imag(lo3),imag(lo4));
    s.p1=s.p1+i*out1;s.p2=s.p2+i*out2;s.p3=s.p3+i*out3;s.p4=s.p4+i*out4;
    c=class(s,'qd');
end
