function c=prod(a,b)
%prod: multiplies elements of the first matrix/vector, of class qd, producing another element of class qd

%  PROD Product of elements.
%     For vectors, PROD(X) is the product of the elements of X. For
%     matrices, PROD(X) is a row vector with the product over each
%     column. For N-D arrays, PROD(X) operates on the first
%     non-singleton dimension.
%  
%     PROD(X,DIM) works along the dimension DIM. 
%  
%     Example: If X = [0 1 2
%                      3 4 5]
%  
%     then prod(X,1) is [0 4 10] and prod(X,2) is [ 0
%                                                  60]
%  
%     See also SUM, CUMPROD, DIFF.
% 
%second parameter, if present, deals with the non-singleton (?) dimension. It is definitely not a matrix
if ~isa(a,'qd')
    a=qd(a);
end
%default is to sum by columns
if nargin<2,b=1;end

%second argument should be an integer; however, convert it to a qd-type just in case
if ~isa(b,'qd')
    b=qd(b);
end
%check dimensions
hi2=getp1(b);
if prod(size(hi2)) == 1
    %invoke the standard function, using the library
    [out1, out2, out3, out4] = qd_prod(getp1(a),getp2(a),getp3(a),getp4(a),getp1(b),getp2(b),getp3(b),getp4(b));
    %reassemble the output matrix from its parts
    %renormalization is not needed because it is already done in the C library
    s.p1=out1;s.p2=out2;s.p3=out3;s.p4=out4;
    c=class(s,'qd');
else
    %error!
    error(['in prod.m; matrices should be have the right size' ])
end
