#include <mex.h>
#include "qd.h"
#include "qd_inline.h"


/*The gateway routine */
void mexFunction(int nlhs,mxArray *plhs [],	int nrhs,const mxArray *prhs [])
	{
	mxLogical *ptrBool;
 	double *inp1, *inp2, *inp3, *inp4;
	double leadingDim;
	int status,mrows,ncols,dummyInt;

	/*Check for proper number of arguments.*/
	/*NOTE:You do not need an else statement when using
	mexErrMsgTxt within an if statement.It will never
	get to the else statement if mexErrMsgTxt is executed.
	(mexErrMsgTxt breaks you out of the MEX-file.)
	*/
	if (nrhs !=5)
		mexErrMsgTxt("Five inputs required.");
	if (nlhs !=1)
		mexErrMsgTxt("Just one output required.");


	if (!mxIsDouble(prhs [0 ]) || mxIsComplex(prhs [0 ]) || !mxIsDouble(prhs [1 ]) || mxIsComplex(prhs [1 ])) {
		mexErrMsgTxt("Input matrix type are not suitable.");
	}

	/*Create a pointer to the input matrix.*/
 	inp1 =mxGetPr(prhs [0 ]);
 	inp2 =mxGetPr(prhs [1 ]);
 	inp3 =mxGetPr(prhs [2 ]);
 	inp4 =mxGetPr(prhs [3 ]);

	/*Get the fifth input, a scalar*/
	leadingDim =mxGetScalar(prhs [4 ]);
	
	/*Get the dimensions of the matrix input.*/
	mrows =mxGetM(prhs [0 ]);
	ncols =mxGetN(prhs [0 ]);
	
	//standard case  is leadingDim=1 ==> gives back a row 
	if (leadingDim==1){
		plhs [0 ] = mxCreateLogicalMatrix(1,ncols);
		
		/*Create a C pointer to a copy of the output matrix.*/
		ptrBool = mxGetLogicals(plhs [0 ]);
		
		/*Call the C subroutine.*/
		int i, j;
		for ( j=0; j<ncols; j++) {
			dummyInt = 0;
			for (i=0; i<mrows; i++) {
				/* index i+ncols*j ranges from 0 to (mrows-1)+ncols*(ncols-1)*/
				qd_real a(inp1[i+mrows*j], inp2[i+mrows*j], inp3[i+mrows*j], inp4[i+mrows*j]);			
				qd_real zero(0,0,0,0);
				//check if a is less, equal or larger than zero (which is equivalent to -1, 0 , 1)
				if (a < zero)
					dummyInt = dummyInt+1;
				else if (a > zero)
					dummyInt = dummyInt+1;
				else 
					dummyInt = 0;
				//According to matlab, nan counts as NonZero so automatically
				//is considered				
			}
			//initially is created as false
			if (dummyInt==mrows) {
				*( mxGetLogicals(plhs[0]) + j )=true;
			}
		}
	}
	if (leadingDim==2){
		plhs [0 ] = mxCreateLogicalMatrix(mrows,1);
		
		/*Create a C pointer to a copy of the output matrix.*/
		ptrBool = mxGetLogicals(plhs [0 ]);
		
		/*Call the C subroutine.*/
		int i, j;
		for (i=0; i<mrows; i++) {
			dummyInt = 0;
			for ( j=0; j<ncols; j++) {
				/* index i+ncols*j ranges from 0 to (mrows-1)+ncols*(ncols-1)*/
				qd_real a(inp1[i+mrows*j], inp2[i+mrows*j], inp3[i+mrows*j], inp4[i+mrows*j]);			
				qd_real zero(0,0,0,0);
				//check if a is less, equal or larger than zero (which is equivalent to -1, 0 , 1)
				if (a < zero)
					dummyInt = dummyInt+1;
				else if (a > zero)
					dummyInt = dummyInt+1;
				else 
					dummyInt = 0;
				//According to matlab, nan counts as NonZero so automatically
				//is considered
			}
			
			//initially is created as false
			if (dummyInt==ncols) {
				*( mxGetLogicals(plhs[0]) + i)=true;
			}
		}
	}

}