#include <mex.h>
#include "qd.h"
#include "qd_inline.h"


/*The gateway routine */
void mexFunction(int nlhs,mxArray *plhs [],	int nrhs,const mxArray *prhs [])
	{
	mxLogical *ptrBool;
 	double *inp1, *inp2, *inp3, *inp4;
	double leadingDim;

	int status,mrows,ncols,dummyInt;

	/*Check for proper number of arguments.*/
	/*NOTE:You do not need an else statement when using
	mexErrMsgTxt within an if statement.It will never
	get to the else statement if mexErrMsgTxt is executed.
	(mexErrMsgTxt breaks you out of the MEX-file.)
	*/
	if (nrhs !=5)
		mexErrMsgTxt("Five inputs required.");
	if (nlhs !=1)
		mexErrMsgTxt("Just one output required.");


	if (!mxIsDouble(prhs [0 ]) || mxIsComplex(prhs [0 ]) || !mxIsDouble(prhs [1 ]) || mxIsComplex(prhs [1 ])) {
		mexErrMsgTxt("Input matrix type are not suitable.");
	}

	/*Create a pointer to the input matrix.*/
 	inp1 =mxGetPr(prhs [0 ]);
 	inp2 =mxGetPr(prhs [1 ]);
 	inp3 =mxGetPr(prhs [2 ]);
 	inp4 =mxGetPr(prhs [3 ]);

	/*Get the third input, a scalar*/
	leadingDim =mxGetScalar(prhs [2 ]);

	/*Get the dimensions of the matrix input.*/
	mrows =mxGetM(prhs [0 ]);
	ncols =mxGetN(prhs [0 ]);

	//standard case  is leadingDim=1 ==> gives back a row 
	if (leadingDim==1){
		plhs [0 ] = mxCreateLogicalMatrix(1,ncols);
		
		/*Create a C pointer to a copy of the output matrix.*/
		ptrBool = mxGetLogicals(plhs [0 ]);
		
		/*Call the C subroutine.*/
		int i, j;
		for ( j=0; j<ncols; j++) {
			for (i=0; i<mrows; i++) {
				/* index i+ncols*j ranges from 0 to (mrows-1)+ncols*(ncols-1)*/
				qd_real a(inp1[i+mrows*j], inp2[i+mrows*j], inp3[i+mrows*j], inp4[i+mrows*j]);			
				qd_real zero(0,0,0,0);
				//check if a is less, equal or larger than zero (which is equivalent to -1, 0 , 1)
				if (a < zero)
					dummyInt = 1;
				else if (a > zero)
					dummyInt = 1;
				else 
					dummyInt = 0;
				
				//initially is created as false
				if (dummyInt==1 & ~mxIsNaN(a[0])) {
					*( mxGetLogicals(plhs[0]) + j )=true;
				}
			}
		}
	}
	if (leadingDim==2){
		plhs [0 ] = mxCreateLogicalMatrix(mrows,1);
		
		/*Create a C pointer to a copy of the output matrix.*/
		ptrBool = mxGetLogicals(plhs [0 ]);
		
		/*Call the C subroutine.*/
		int i, j;
		for ( j=0; j<ncols; j++) {
			for (i=0; i<mrows; i++) {
				/* index i+ncols*j ranges from 0 to (mrows-1)+ncols*(ncols-1)*/
   			    qd_real a(inp1[i+mrows*j], inp2[i+mrows*j], inp3[i+mrows*j], inp4[i+mrows*j]);			
                qd_real zero(0,0,0,0);
				//check if a is less, equal or larger than zero (which is equivalent to -1, 0 , 1)
				if (a < zero)
					dummyInt = 1;
				else if (a > zero)
					dummyInt = 1;
				else 
					dummyInt = 0;
				//Automatically considers the case of NaN's

				//initially is created as false
				if (dummyInt==1) {
					*( mxGetLogicals(plhs[0]) + i)=true;
				}
			}
		}
	}

}