#include <mex.h>
#include "qd.h"
#include "qd_inline.h"


/*The gateway routine */
void mexFunction(int nlhs,mxArray *plhs [],	int nrhs,const mxArray *prhs [])
{
	double *ptrI, *ptrJ, *ptrValuesP1, *ptrValuesP2, *ptrValuesP3, *ptrValuesP4;
	double *inp1, *inp2, *inp3, *inp4, *ValuesP1, *ValuesP2, *ValuesP3, *ValuesP4, *indexI, *indexJ;
	int status,mrows,ncols,nnz,dummyInt;
	
	/*Check for proper number of arguments.*/
	/*NOTE:You do not need an else statement when using
	mexErrMsgTxt within an if statement.It will never
	get to the else statement if mexErrMsgTxt is executed.
	(mexErrMsgTxt breaks you out of the MEX-file.)
	*/
	if (nrhs !=4)
		mexErrMsgTxt("Four inputs required.");
	if (nlhs !=4)
		mexErrMsgTxt("Four outputs required.");
	
	
	if (!mxIsDouble(prhs [0 ]) || mxIsComplex(prhs [0 ]) || !mxIsDouble(prhs [1 ]) || mxIsComplex(prhs [1 ])) {
		mexErrMsgTxt("Input matrix type are not suitable.");
	}
	
	/*Create a pointer to the input matrices.*/
 	inp1 =mxGetPr(prhs [0 ]);
 	inp2 =mxGetPr(prhs [1 ]);
 	inp3 =mxGetPr(prhs [2 ]);
 	inp4 =mxGetPr(prhs [3 ]);
	
	/*Get the dimensions of the first matrix input.*/
	mrows =mxGetM(prhs [0 ]);
	ncols =mxGetN(prhs [0 ]);
	
	//allocate enough space for the arrays
	indexI = (double*) malloc(mrows*ncols*sizeof(double));
	indexJ = (double*) malloc(mrows*ncols*sizeof(double));
	
	
	/*Call the C subroutine.*/
	ValuesP1 = (double*) malloc(mrows*ncols*sizeof(double));
	ValuesP2 = (double*) malloc(mrows*ncols*sizeof(double));
	ValuesP3 = (double*) malloc(mrows*ncols*sizeof(double));
	ValuesP4 = (double*) malloc(mrows*ncols*sizeof(double));

	int i, j;
	nnz=0;
	for ( j=0; j<ncols; j++) {
		for (i=0; i<mrows; i++) {
			/* index i+ncols*j ranges from 0 to (mrows-1)+ncols*(ncols-1)*/
			qd_real a(inp1[i+mrows*j], inp2[i+mrows*j], inp3[i+mrows*j], inp4[i+mrows*j]);			
			qd_real cero(0,0,0,0);
			//check if a is less, equal or larger than zero(which is equivalent to -1, 0 , 1)
			if (a < cero)
				dummyInt = 1;
			else if (a > cero)
				dummyInt = 1;
			else 
				dummyInt = 0;
			
			if (dummyInt==1 & ~mxIsNaN(a.x[0])) {
				ValuesP1[nnz]=inp1[i+mrows*j];
				ValuesP2[nnz]=inp2[i+mrows*j];
				ValuesP3[nnz]=inp3[i+mrows*j];
				ValuesP4[nnz]=inp4[i+mrows*j];
				indexI[nnz]=i;
				indexJ[nnz]=j;
				nnz=nnz+1;//counter for the NonZero entries
			}
		}
	}
	
	/*Set the output pointer to the output row matrix.
	The total number of entries now is known, and is equal to nnz-1*/
	plhs [0 ] = mxCreateDoubleMatrix(nnz,1,mxREAL);
	plhs [1 ] = mxCreateDoubleMatrix(nnz,1,mxREAL);
	
	
	/*Create a C pointer to a copy of the output vectors.*/
	ptrI =mxGetPr(plhs [0 ]);
	ptrJ =mxGetPr(plhs [1 ]);
	
	plhs [2 ] = mxCreateDoubleMatrix(nnz,1,mxREAL);
	plhs [3 ] = mxCreateDoubleMatrix(nnz,1,mxREAL);
	plhs [4 ] = mxCreateDoubleMatrix(nnz,1,mxREAL);
	plhs [5 ] = mxCreateDoubleMatrix(nnz,1,mxREAL);

	//pointers to the x[1],x[2], etc. elements for the output 
	ptrValuesP1 =mxGetPr(plhs [2 ]);
	ptrValuesP2 =mxGetPr(plhs [3 ]);
	ptrValuesP3 =mxGetPr(plhs [4 ]);
	ptrValuesP4 =mxGetPr(plhs [5 ]);
	for ( j=0; j<=nnz; j++) {
		ptrValuesP1[j]=ValuesP1[j];
		ptrValuesP2[j]=ValuesP2[j];
		ptrValuesP3[j]=ValuesP3[j];
		ptrValuesP4[j]=ValuesP4[j];
		ptrI[j]=1+indexI[j];
		ptrJ[j]=1+indexJ[j];
	}
}
