#include <mex.h>
#include "qd.h"
#include "qd_inline.h"


/*The gateway routine */
void mexFunction(int nlhs,mxArray *plhs [],	int nrhs,const mxArray *prhs [])
	{
 	double *inp1, *inp2, *inp3, *inp4;
 	double *sec1, *sec2, *sec3, *sec4;
 	double *out1, *out2, *out3, *out4;
	int status,mrows,ncols,mrows2,ncols2;

	/*Check for proper number of arguments.*/
	/*NOTE:You do not need an else statement when using
	mexErrMsgTxt within an if statement.It will never
	get to the else statement if mexErrMsgTxt is executed.
	(mexErrMsgTxt breaks you out of the MEX-file.)
	*/
	if (nrhs !=8)
		mexErrMsgTxt("Eight inputs required.");
	if (nlhs !=4)
		mexErrMsgTxt("Four output required.");


	if (!mxIsDouble(prhs [0 ]) || mxIsComplex(prhs [0 ]) || !mxIsDouble(prhs [1 ]) || mxIsComplex(prhs [1 ] ) ) {
		mexErrMsgTxt("Input data type is not allowed.");
	}

	/*check matrices dimensions */
	if (mxGetN(prhs [0 ]) != mxGetM(prhs [2 ] ) ){
		mexErrMsgTxt("Input matrix dimensions must agree.");
	}

	/*Create a pointer to the input matrices.*/
 	inp1 =mxGetPr(prhs [0 ]);
 	inp2 =mxGetPr(prhs [1 ]);
 	inp3 =mxGetPr(prhs [2 ]);
 	inp4 =mxGetPr(prhs [3 ]);
 	sec1 =mxGetPr(prhs [4 ]);
 	sec2 =mxGetPr(prhs [5 ]);
 	sec3 =mxGetPr(prhs [6 ]);
 	sec4 =mxGetPr(prhs [7 ]);

	/*Get the dimensions of the first matrix input.
	  Second matrix should be equal*/
	mrows =mxGetM(prhs [0 ]);
	ncols =mxGetN(prhs [0 ]);

	/*Get the dimensions of the third matrix input.
	  Fourth matrix should be equal*/
	mrows2 =mxGetM(prhs [2 ]);
	ncols2 =mxGetN(prhs [2 ]);
    /*ncols should be equal to mrows2; it has been checked in the calling routine*/

	/*Set the output pointer to the output matrix.*/
	plhs [0 ] = mxCreateDoubleMatrix(mrows,ncols2,mxREAL);
	plhs [1 ] = mxCreateDoubleMatrix(mrows,ncols2,mxREAL);
	plhs [2 ] = mxCreateDoubleMatrix(mrows,ncols2,mxREAL);
	plhs [3 ] = mxCreateDoubleMatrix(mrows,ncols2,mxREAL);

	/*Create a C pointer to a copy of the output matrix.*/
 	out1 =mxGetPr(plhs [0 ]);
 	out2 =mxGetPr(plhs [1 ]);
 	out3 =mxGetPr(plhs [2 ]);
 	out4 =mxGetPr(plhs [3 ]);

	/*Call the C subroutine.*/

	int i, j, k;
	for (i=0; i<mrows; i++) {
		for ( j=0; j<ncols2; j++) {
			qd_real cum(0,0,0,0);
			for ( k=0; k<ncols; k++) {
				/* index ranges from 0 to mrows*ncols - 1 = (mrows-1)+mrows*(ncols-1)*/
				qd_real a(inp1[i+mrows*k], inp2[i+mrows*k], inp3[i+mrows*k], inp4[i+mrows*k]);
				/* index ranges from 0 to mrows2*ncols2 - 1 = (ncols-1)+mrows2*(ncols2-1) = (mrows2-1)+mrows2*(ncols2-1)*/
				qd_real b(sec1[k+j*mrows2], sec2[k+j*mrows2],sec3[k+j*mrows2], sec4[k+j*mrows2]);
				
				cum=cum+a*b;
			}
			/* index ranges from 0 to mrows*ncols2 - 1 = (ncols2-1)+ncols2*(mrows-1)*/
			out1[j+i*ncols2] = cum.x[0];
			out2[j+i*ncols2] = cum.x[1];
			out3[j+i*ncols2] = cum.x[2];
			out4[j+i*ncols2] = cum.x[3];			
		}
	}
}

