#include <mex.h>
#include "qd.h"
#include "qd_inline.h"


/*The gateway routine */
void mexFunction(int nlhs,mxArray *plhs [],	int nrhs,const mxArray *prhs [])
	{
 	double *inp1, *inp2, *inp3, *inp4;
 	double *sec1, *sec2, *sec3, *sec4;
 	double *out1, *out2, *out3, *out4;
	int status,mrows,ncols;

	/*Check for proper number of arguments.*/
	/*NOTE:You do not need an else statement when using
	mexErrMsgTxt within an if statement.It will never
	get to the else statement if mexErrMsgTxt is executed.
	(mexErrMsgTxt breaks you out of the MEX-file.)
	*/
	if (nrhs !=8)
		mexErrMsgTxt("Eight inputs required.");
	if (nlhs !=4)
		mexErrMsgTxt("Four output required.");


	if (!mxIsDouble(prhs [0 ]) || mxIsComplex(prhs [0 ]) || !mxIsDouble(prhs [1 ]) || mxIsComplex(prhs [1 ]) || mxGetN(prhs [0 ]) != mxGetN(prhs [1 ] )  || mxGetM(prhs [0 ]) != mxGetM(prhs [1 ] ) ){
		mexErrMsgTxt("Input matrix dimensions must agree.");
	}

	/*Create a pointer to the input matrices.*/
 	inp1 =mxGetPr(prhs [0 ]);
 	inp2 =mxGetPr(prhs [1 ]);
 	inp3 =mxGetPr(prhs [2 ]);
 	inp4 =mxGetPr(prhs [3 ]);
 	sec1 =mxGetPr(prhs [4 ]);
 	sec2 =mxGetPr(prhs [5 ]);
 	sec3 =mxGetPr(prhs [6 ]);
 	sec4 =mxGetPr(prhs [7 ]);

	/*Get the dimensions of the first matrix input y.*/
	mrows =mxGetM(prhs [1 ]);
	ncols =mxGetN(prhs [1 ]);

	/*Set the output pointer to the output matrix.*/
 	plhs [0 ] = mxCreateDoubleMatrix(mrows,ncols,mxREAL);
 	plhs [1 ] = mxCreateDoubleMatrix(mrows,ncols,mxREAL);
 	plhs [2 ] = mxCreateDoubleMatrix(mrows,ncols,mxREAL);
 	plhs [3 ] = mxCreateDoubleMatrix(mrows,ncols,mxREAL);
 
 	/*Create a C pointer to a copy of the output matrix.*/
 	out1 =mxGetPr(plhs [0 ]);
 	out2 =mxGetPr(plhs [1 ]);
 	out3 =mxGetPr(plhs [2 ]);
 	out4 =mxGetPr(plhs [3 ]);

	/*Call the C subroutine.*/
	double inf,nan;
	//positive infinite according to IEEE
	inf=mxGetInf();
	//Not a Number, according to IEEE
    nan=mxGetNaN();
	int i, j;
	for (i=0; i<mrows; i++) {
		for ( j=0; j<ncols; j++) {
			qd_real a(inp1[i+mrows*j], inp2[i+mrows*j], inp3[i+mrows*j], inp4[i+mrows*j]);			
			qd_real b(sec1[i+mrows*j], sec2[i+mrows*j], sec3[i+mrows*j], sec4[i+mrows*j]);	
			
			/*deal with infinites, nan, etc.
			Rules:
			Inf+x=Inf; x not a NaN, not a Inf
			Inf+NaN=NaN
			Inf-Inf=NaN 
			NaN+NaN=NaN
			Tests to be performed in matlab:
			[a,b]=qd_plus( Inf,   0, Inf,   0);[ Inf  Inf a b]
			[a,b]=qd_plus( Inf, Inf,   0,   0);[ Inf  Inf a b]
			[a,b]=qd_plus( Inf, Inf, Inf, Inf);[ Inf  Inf a b]
			[a,b]=qd_plus(-Inf,   0,   0,   0);[-Inf -Inf a b]
			[a,b]=qd_plus( Inf,-Inf,   0,   0);[ Inf  Inf a b] %because hi dominates lo
			[a,b]=qd_plus( Inf,   0,-Inf,   0);[ NaN  NaN a b]
			[a,b]=qd_plus( Inf, NaN, Inf, Inf);[ NaN  NaN a b]
			[a,b]=qd_plus(-Inf,   0, Inf,   0);[ NaN  NaN a b]
			[a,b]=qd_plus(   1,   2, NaN,   3);[ NaN  NaN a b]
			[a,b]=qd_plus(   1,   0,   2,   0);[   3    0 a b]
			ahi=[Inf;Inf;Inf;-Inf;Inf;Inf;Inf;-Inf;1;1]
			alo=[0;Inf;Inf;0;-Inf;0;NaN;0;NaN;0]
			bhi=[Inf;0;Inf;0;0;-Inf;Inf;Inf;NaN;2]
			blo=[0;0;Inf;0;0;0;Inf;0;3;0]
			shouldBe=[Inf Inf;Inf Inf;Inf Inf;-Inf -Inf;Inf Inf;NaN NaN;NaN NaN;NaN NaN;NaN NaN;3 0]
			[a,b]=qd_plus(ahi,alo,bhi,blo);[shouldBe a b]
			*/
 
			//check if needs to worry about NaN's and Inf's
			if ( mxIsNaN(a.x[0]) | mxIsNaN(a.x[1]) | mxIsNaN(a.x[2]) | mxIsNaN(a.x[3]) | mxIsInf(a.x[0]) | mxIsInf(a.x[1]) | mxIsInf(a.x[2]) | mxIsInf(a.x[3]) | \
			mxIsNaN(b.x[0]) | mxIsNaN(b.x[1]) | mxIsNaN(b.x[2]) | mxIsNaN(b.x[3]) | mxIsInf(b.x[0]) | mxIsInf(b.x[1]) | mxIsInf(b.x[2]) | mxIsInf(b.x[3])) {
				if ( mxIsNaN(a.x[0]) | mxIsNaN(a.x[1]) | mxIsNaN(a.x[2]) | mxIsNaN(a.x[3]) | mxIsNaN(b.x[0]) | mxIsNaN(b.x[1]) | mxIsNaN(b.x[2]) | mxIsNaN(b.x[3]) | \
				(mxIsInf(a.x[0]) & mxIsInf(b.x[0]) & (a.x[0]*b.x[0]<0)) | (mxIsInf(a.x[0]) & mxIsInf(b.x[0]) & (a.x[0]*b.x[0]<0))) { 
					//the output should be NaN
					out1[i+mrows*j] = nan;
					out2[i+mrows*j] = nan;	
					out3[i+mrows*j] = nan;
					out4[i+mrows*j] = nan;	
					continue;
				}
				else{
					//The output is some kind of infinite
					//Both inputs are infinites
					if ((mxIsInf(a.x[0]) & mxIsInf(b.x[0]) & (a.x[0]*b.x[0]>=0))) {
						out1[i+mrows*j] = a.x[0];
						out2[i+mrows*j] = a.x[1];
						out3[i+mrows*j] = a.x[2];
						out4[i+mrows*j] = a.x[3];
						continue;
					}
					//At least one is infinite
					if (mxIsInf(a.x[0]) & !mxIsInf(b.x[0])) {
						out1[i+mrows*j] = a.x[0];
						out2[i+mrows*j] = a.x[1];
						out3[i+mrows*j] = a.x[2];
						out4[i+mrows*j] = a.x[3];
						continue;
					}
					if (!mxIsInf(a.x[0]) & mxIsInf(b.x[0])) {
						out1[i+mrows*j] = b.x[0];
						out2[i+mrows*j] = b.x[1];
						out3[i+mrows*j] = b.x[2];
						out4[i+mrows*j] = b.x[3];
						continue;
					}
				}
			}
			else{
				//standard case
				qd_real c;	
				c=a+b;
				out1[i+mrows*j] = c.x[0];
				out2[i+mrows*j] = c.x[1];
				out3[i+mrows*j] = c.x[2];
				out4[i+mrows*j] = c.x[3];
			}
		}
	}
}

