 #include <mex.h>
 #include "qd.h"
 #include "qd_inline.h"
 
 
 /*The gateway routine */
 void mexFunction(int nlhs,mxArray *plhs [],	int nrhs,const mxArray *prhs [])
 	{
 	double *inp1, *inp2, *inp3, *inp4;
 	double *sec1, *sec2, *sec3, *sec4;
 	double *out1, *out2, *out3, *out4;
 	int status,mrows,ncols;
 
 	/*Check for proper number of arguments.*/
 	/*NOTE:You do not need an else statement when using
 	mexErrMsgTxt within an if statement.It will never
 	get to the else statement if mexErrMsgTxt is executed.
 	(mexErrMsgTxt breaks you out of the MEX-file.)
 	*/
 	if (nrhs !=8)
 		mexErrMsgTxt("Eight inputs required.");
 	if (nlhs !=4)
 		mexErrMsgTxt("Four output required.");
 
 
 	if (!mxIsDouble(prhs [0 ]) || mxIsComplex(prhs [0 ]) || !mxIsDouble(prhs [1 ]) || mxIsComplex(prhs [1 ]) ){
 		mexErrMsgTxt("Input matrix type should be ok.");
 	}
 
 	/*Create a pointer to the input matrices.*/
 	inp1 =mxGetPr(prhs [0 ]);
 	inp2 =mxGetPr(prhs [1 ]);
 	inp3 =mxGetPr(prhs [2 ]);
 	inp4 =mxGetPr(prhs [3 ]);
 	sec1 =mxGetPr(prhs [4 ]);
 	sec2 =mxGetPr(prhs [5 ]);
 	sec3 =mxGetPr(prhs [6 ]);
 	sec4 =mxGetPr(prhs [7 ]);
 
 	/*Get the dimensions of the first matrix input y.*/
 	mrows =mxGetM(prhs [0 ]);
 	ncols =mxGetN(prhs [0 ]);
 
 	/*Set the output pointer to the output matrix.*/
 	plhs [0 ] = mxCreateDoubleMatrix(mrows,ncols,mxREAL);
 	plhs [1 ] = mxCreateDoubleMatrix(mrows,ncols,mxREAL);
 	plhs [2 ] = mxCreateDoubleMatrix(mrows,ncols,mxREAL);
 	plhs [3 ] = mxCreateDoubleMatrix(mrows,ncols,mxREAL);
 
 	/*Create a C pointer to a copy of the output matrix.*/
 	out1 =mxGetPr(plhs [0 ]);
 	out2 =mxGetPr(plhs [1 ]);
 	out3 =mxGetPr(plhs [2 ]);
 	out4 =mxGetPr(plhs [3 ]);
 
 	/*Call the C subroutine.*/
 
 	int i, j;
	double inf, nan;
	inf=mxGetInf();
	nan=mxGetNaN();

 	for (i=0; i<mrows; i++) {
 		for ( j=0; j<ncols; j++) {
 
			qd_real a(inp1[i+mrows*j], inp2[i+mrows*j], inp3[i+mrows*j], inp4[i+mrows*j]);			
			qd_real b(sec1[i+mrows*j], sec2[i+mrows*j], sec3[i+mrows*j], sec4[i+mrows*j]);	
 			qd_real c, maxexp(709,0,0,0), zero(0,0,0,0), one(1,0,0,0), two(2,0,0,0);
 			//c=a^b;
			if  (a<=zero){
				//check for an integer exponent
				if (a==zero) {
					if (b>zero) {
						//the exponent itself doesn't matter
						out1[i+mrows*j] = 0;
						out2[i+mrows*j] = 0;
						out3[i+mrows*j] = 0;
						out4[i+mrows*j] = 0;
						continue;
					}
					else{
						if (b==zero) {
							out1[i+mrows*j] = nan;
							out2[i+mrows*j] = nan;
							out3[i+mrows*j] = nan;
							out4[i+mrows*j] = nan;
							continue;
						}
						else {//b<0
							out1[i+mrows*j] = inf;
							out2[i+mrows*j] = inf;
							out3[i+mrows*j] = inf;
							out4[i+mrows*j] = inf;
							continue;
						}
					}
				}
				else {//a<0
					if (b==two*nint(b/two)) {
						//a < 0, b=2*n
						//mexPrintf("%f es par\n",b.hi);
						c=b*log(-a);//Check if the result will overflow
						if (c>=maxexp) {
							out1[i+mrows*j] = inf;
							out2[i+mrows*j] = inf;
							out3[i+mrows*j] = inf;
							out4[i+mrows*j] = inf;
							continue;
						}
						else {
							c=npwr(-a,b);
							out1[i+mrows*j] = c.x[0];
							out2[i+mrows*j] = c.x[1];
							out3[i+mrows*j] = c.x[2];
							out4[i+mrows*j] = c.x[3];
							continue;
						}
					}
					else{
						if ((b+one)==two*nint((b+one)/two)) {
							//a < 0, b=2*n+1
							//mexPrintf("%i es impar\n",b.hi);
							c=b*log(-a);//Check if the result will overflow
							if (c>=maxexp) {
								qd_real infinite(-inf,-inf,-inf,-inf);
								c=infinite;
							}
							else {
								c=npwr(-a,b);
								out1[i+mrows*j] = -c.x[0];
								out2[i+mrows*j] = -c.x[1];
								out3[i+mrows*j] = -c.x[2];
								out4[i+mrows*j] = -c.x[3];
								continue;
							}
						}
						else {
							// a<0, and non-integer b ==> NaN's
							out1[i+mrows*j] = nan;
							out2[i+mrows*j] = nan;
							out3[i+mrows*j] = nan;
							out4[i+mrows*j] = nan;
							continue;
						}
					}
				}
			}
			else{ //a>0;  c=pow(a,b);
				//mexPrintf("b=%f, a=%f\n",a.hi,b.hi);
				c=b*log(a);
				//mexPrintf("b=%f, a=%f, c=%f\n",a.hi,b.hi,c.hi);
				if (c>=maxexp) {
					//mexPrintf("c es mayor que 709, i=%i, b=%f, a=%f, c=%f\n",j, a.hi,b.hi,c.hi);
					out1[i+mrows*j] = inf;
					out2[i+mrows*j] = inf; 
					out3[i+mrows*j] = inf;
					out4[i+mrows*j] = inf; 
				}
				else {
					//mexPrintf("c es menor que 709, i=%i, b=%f, a=%f, c=%f\n",j, a.hi,b.hi,c.hi);
					c=exp(c);				
					out1[i+mrows*j] = c.x[0];
					out2[i+mrows*j] = c.x[1]; 
					out3[i+mrows*j] = c.x[2];
					out4[i+mrows*j] = c.x[3]; 
				} 
			}
 		}
 	}
 }
