#include <mex.h>
#include "qd.h"
#include "qd_inline.h"


/*The gateway routine */
void mexFunction(int nlhs,mxArray *plhs [],	int nrhs,const mxArray *prhs [])
{
 	double *inp1, *inp2, *inp3, *inp4;
 	double *sec1, *sec2, *sec3, *sec4;
 	double *out1, *out2, *out3, *out4;
	int status,mrows,ncols;
	
	/*Check for proper number of arguments.*/
	/*NOTE:You do not need an else statement when using
	mexErrMsgTxt within an if statement.It will never
	get to the else statement if mexErrMsgTxt is executed.
	(mexErrMsgTxt breaks you out of the MEX-file.)
	*/
	if (nrhs !=8)
		mexErrMsgTxt("Eight inputs required.");
	if (nlhs !=4)
		mexErrMsgTxt("Four output required.");
	
	
	if (!mxIsDouble(prhs [0 ]) || mxIsComplex(prhs [0 ]) || !mxIsDouble(prhs [1 ]) || mxIsComplex(prhs [1 ]) ){
		mexErrMsgTxt("Input matrix type should be ok.");
	}
	
	/*Create a pointer to the input matrices.*/
 	inp1 =mxGetPr(prhs [0 ]);
 	inp2 =mxGetPr(prhs [1 ]);
 	inp3 =mxGetPr(prhs [2 ]);
 	inp4 =mxGetPr(prhs [3 ]);
 	sec1 =mxGetPr(prhs [4 ]);
 	sec2 =mxGetPr(prhs [5 ]);
 	sec3 =mxGetPr(prhs [6 ]);
 	sec4 =mxGetPr(prhs [7 ]);
	
	/*Get the dimensions of the first matrix input y.*/
	mrows =mxGetM(prhs [0 ]);
	ncols =mxGetN(prhs [0 ]);
	
	/*Set the output pointer to the output matrix.*/
 	plhs [0 ] = mxCreateDoubleMatrix(mrows,ncols,mxREAL);
 	plhs [1 ] = mxCreateDoubleMatrix(mrows,ncols,mxREAL);
 	plhs [2 ] = mxCreateDoubleMatrix(mrows,ncols,mxREAL);
 	plhs [3 ] = mxCreateDoubleMatrix(mrows,ncols,mxREAL);
 
 	/*Create a C pointer to a copy of the output matrix.*/
 	out1 =mxGetPr(plhs [0 ]);
 	out2 =mxGetPr(plhs [1 ]);
 	out3 =mxGetPr(plhs [2 ]);
 	out4 =mxGetPr(plhs [3 ]);
	
	/*Call the C subroutine.*/
	
	int i, j;
	double inf,nan;
	inf=mxGetInf();
	nan=mxGetNaN();
	for (i=0; i<mrows; i++) {
		for ( j=0; j<ncols; j++) {
			qd_real a(inp1[i+mrows*j], inp2[i+mrows*j], inp3[i+mrows*j], inp4[i+mrows*j]);			
			qd_real b(sec1[i+mrows*j], sec2[i+mrows*j], sec3[i+mrows*j], sec4[i+mrows*j]);	
			qd_real c,zero(0,0,0,0);	
//					mexPrintf("(i,j)=(%i,%i): %f : %f\n",i,j,inp1[i+mrows*j],sec1[i+mrows*j]);
			/*deal with infinites, nan, etc.
			Rules:
			(+Inf)/x=+Inf*sign(x); x not a NaN or Inf;
			(*Inf)/(*Inf)=NaN; irrespective of * 
			0/0=NaN
			0/Inf=0
			x/Nan=NaN for any x 
			NaN/x=NaN for any x
			
			  %Tests to be performed in matlab:
			  [a,b,c,d]=qd_rdivide(1,0,0,0);[Inf Inf a b]
			  [a,b,c,d]=qd_rdivide(-1,0,0,0);[-Inf -Inf a b]
			  [a,b,c,d]=qd_rdivide(+Inf,+Inf,0,0);[Inf Inf a b]
			  [a,b,c,d]=qd_rdivide(Inf,Inf,Inf,Inf);[NaN NaN a b]
			  [a,b,c,d]=qd_rdivide(-Inf,-Inf,2,2);[-Inf -Inf a b]
			  [a,b,c,d]=qd_rdivide(0,0,0,0);[NaN NaN a b]
			  [a,b,c,d]=qd_rdivide(NaN,NaN,2,2);[NaN NaN a b] 
			*/
			
			//check if needs to worry about NaN's and Inf's
			if ( mxIsNaN(a.x[0]) | mxIsNaN(b.x[0]) | mxIsInf(a.x[0]) | mxIsInf(b.x[0]) | (a==zero & b==zero)){
				if ((mxIsNaN(a.x[0]) | mxIsNaN(b.x[0])) | (mxIsInf(a.x[0]) & mxIsInf(b.x[0])) | (a==zero & b==zero)) { 
					//One input is nan, or both inputs are Inf, or both are zeros==>the output should be NaN
					out1[i+mrows*j] = nan;
					out2[i+mrows*j] = nan;	
					out3[i+mrows*j] = nan;
					out4[i+mrows*j] = nan;	
					continue;
				}
				else{
					//Only one of the inputs is infinite
					double signo;
					signo=-1;
					if (a.x[0]>0) signo=1;
					if (b.x[0]<0) signo=-signo;
					if (mxIsInf(a.x[0])) {
						if (a.x[0]>0 & signo==1) {
							//worry about the a.x[0] part; a.x[i],i>0 are not relevant
							//provided the number is normalized
							out1[i+mrows*j] = inf;
							out2[i+mrows*j] = inf;
							out3[i+mrows*j] = inf;
							out4[i+mrows*j] = inf;
							continue;
						}
						else
						{
							out1[i+mrows*j] = -inf;
							out2[i+mrows*j] = -inf;
							out3[i+mrows*j] = -inf;
							out4[i+mrows*j] = -inf;
							continue;
						}
					}
					else {
						//b is infinite
						out1[i+mrows*j] = 0;
						out2[i+mrows*j] = 0;
						out3[i+mrows*j] = 0;
						out4[i+mrows*j] = 0;
						continue;
					}
				}
			}
			else 
			{
				//this is executed in the regular case
				if (b==zero) {
					if (a.x[0]>0) {
						out1[i+mrows*j] = inf;
						out2[i+mrows*j] = inf;
						out3[i+mrows*j] = inf;
						out4[i+mrows*j] = inf;
						continue;
					}
					else {
						out1[i+mrows*j] = -inf;
						out2[i+mrows*j] = -inf;
						out3[i+mrows*j] = -inf;
						out4[i+mrows*j] = -inf;
						continue;
					}
				}
				else {
					c=a/b;
					out1[i+mrows*j] = c.x[0];
					out2[i+mrows*j] = c.x[1];
					out3[i+mrows*j] = c.x[2];
					out4[i+mrows*j] = c.x[3];
				}
			}			
		}
	}
}