 #include <mex.h>
 #include "qd.h"
 #include "qd_inline.h"
 
 
 /*The gateway routine */
 void mexFunction(int nlhs,mxArray *plhs [],	int nrhs,const mxArray *prhs [])
 	{
 	double *inp1, *inp2, *inp3, *inp4;
 	double *out1, *out2, *out3, *out4;
 	int status,mrows,ncols;
 
 	/*Check for proper number of arguments.*/
 	/*NOTE:You do not need an else statement when using
 	mexErrMsgTxt within an if statement.It will never
 	get to the else statement if mexErrMsgTxt is executed.
 	(mexErrMsgTxt breaks you out of the MEX-file.)
 	*/
 	if (nrhs !=4)
 		mexErrMsgTxt("Four inputs required.");
 	if (nlhs !=4)
 		mexErrMsgTxt("Four output required.");
 
 
 	if (!mxIsDouble(prhs [0 ]) || mxIsComplex(prhs [0 ]) || !mxIsDouble(prhs [1 ]) || mxIsComplex(prhs [1 ]) ){
 		mexErrMsgTxt("Input matrix type should be ok.");
 	}
 
 	/*Create a pointer to the input matrices.*/
 	inp1 =mxGetPr(prhs [0 ]);
 	inp2 =mxGetPr(prhs [1 ]);
 	inp3 =mxGetPr(prhs [2 ]);
 	inp4 =mxGetPr(prhs [3 ]);
 
 	/*Get the dimensions of the first matrix input y.*/
 	mrows =mxGetM(prhs [0 ]);
 	ncols =mxGetN(prhs [0 ]);
 
 	/*Set the output pointer to the output matrix.*/
 	plhs [0 ] = mxCreateDoubleMatrix(mrows,ncols,mxREAL);
 	plhs [1 ] = mxCreateDoubleMatrix(mrows,ncols,mxREAL);
 	plhs [2 ] = mxCreateDoubleMatrix(mrows,ncols,mxREAL);
 	plhs [3 ] = mxCreateDoubleMatrix(mrows,ncols,mxREAL);
 
 	/*Create a C pointer to a copy of the output matrix.*/
 	out1 =mxGetPr(plhs [0 ]);
 	out2 =mxGetPr(plhs [1 ]);
 	out3 =mxGetPr(plhs [2 ]);
 	out4 =mxGetPr(plhs [3 ]);
 
 	/*Call the C subroutine.*/
 
 	int i, j;
	double inf,nan;
	inf=mxGetInf();
	nan=mxGetNaN();
	qd_real zero(0,0,0,0);
 	for (i=0; i<mrows; i++) {
 		for ( j=0; j<ncols; j++) {
            /*use as index [i+mrows*j]
                   minimum taken value is obviously 0
                   maximum value is (mrows-1)+mrows*(ncols-1)=mrows*ncols-1*/
 			qd_real a(inp1[i+mrows*j], inp2[i+mrows*j], inp3[i+mrows*j], inp4[i+mrows*j]);
 			qd_real c;	
			/*deal with infinites, nan, etc.
			Rules:
			sqrt(Inf,x,y,z)=Inf; x not a NaN
			sqrt(-Inf,x,y,z)=NaN <== should be a complex value
			sqrt(-x,*)=NaN <== should be a complex value
			sqrt(NaN,*)=NaN
			Tests to be performed in matlab:
			[a,b]=qd_sqrt(100,  0);[10 0 a b]
			[a,b]=qd_sqrt(+Inf,  0);[Inf Inf a b]
			[a,b]=qd_sqrt( Inf,Inf);[Inf Inf a b]
			[a,b]=qd_sqrt( Inf,-Inf);[Inf Inf a b]
			[a,b]=qd_sqrt(-Inf,+Inf);[NaN NaN a b] % <== should be a complex value
			[a,b]=qd_sqrt(NaN,2);[NaN NaN a b] %should not be a valid combination
			[a,b]=qd_sqrt(NaN,Inf);[NaN NaN a b] %should not be a valid combination
			*/
			
			//check if needs to worry about NaN's and Inf's
			if ( mxIsNaN(a.x[0]) | mxIsNaN(a.x[1]) | mxIsNaN(a.x[2]) | mxIsNaN(a.x[3]) | mxIsInf(a.x[1]) | mxIsInf(a.x[2])| mxIsInf(a.x[3]) | mxIsInf(a.x[4]) ){
				if (mxIsNaN(a.x[0]) | mxIsNaN(a.x[1]) | mxIsNaN(a.x[2]) | mxIsNaN(a.x[3])) { 
					//the output should be NaN
					out1[i+mrows*j] = nan;
					out2[i+mrows*j] = nan;
					out3[i+mrows*j] = nan;
					out4[i+mrows*j] = nan;
					continue;
				}
				else{
					//The input is some kind of infinite
					if (mxIsInf(a.x[0])) {
						if (a>zero) {
							//worry about the a.p1 part; the others are not relevant
							//provided the number is normalized
							out1[i+mrows*j] = a.x[0];
							out2[i+mrows*j] = a.x[0];
							out3[i+mrows*j] = a.x[0];
							out4[i+mrows*j] = a.x[0];
							continue;
						}
						else
						{
							//sqrt(-inf)=0 -inf*i
							//this should be a complex value; put NaN in the meantime
							out1[i+mrows*j] = nan;
							out2[i+mrows*j] = nan;
							out3[i+mrows*j] = nan;
							out4[i+mrows*j] = nan;
							continue;
						}
					}
				}
			}
			else 
			{
				//this is executed in the regular case
				if (a>=zero) {
					c=sqrt(a);
					out1[i+mrows*j] = c.x[0];
					out2[i+mrows*j] = c.x[1];
					out3[i+mrows*j] = c.x[2];
					out4[i+mrows*j] = c.x[3];
				}else{
					//this should be a complex value; put NaN in the meantime
					out1[i+mrows*j] = nan;
					out2[i+mrows*j] = nan;
					out3[i+mrows*j] = nan;
					out4[i+mrows*j] = nan;
				}				
			} 
 		}
 	}
 }
