#include <mex.h>
#include "qd.h"
#include "qd_inline.h"


/*The gateway routine */
void mexFunction(int nlhs,mxArray *plhs [],	int nrhs,const mxArray *prhs [])
	{
 	double *inp1, *inp2, *inp3, *inp4;
 	double *out1, *out2, *out3, *out4;
	double leadingDim;
	int status,mrows,ncols;

	/*Check for proper number of arguments.*/
	/*NOTE:You do not need an else statement when using
	mexErrMsgTxt within an if statement.It will never
	get to the else statement if mexErrMsgTxt is executed.
	(mexErrMsgTxt breaks you out of the MEX-file.)
	*/
	if (nrhs !=5)
		mexErrMsgTxt("Five inputs required.");
	if (nlhs !=4)
		mexErrMsgTxt("Four output required.");


	if (!mxIsDouble(prhs [0 ]) || mxIsComplex(prhs [0 ]) || !mxIsDouble(prhs [1 ]) || mxIsComplex(prhs [1 ])) {
		mexErrMsgTxt("Input matrix type are not suitable.");
	}

	/*Create a pointer to the input matrices.*/
 	inp1 =mxGetPr(prhs [0 ]);
 	inp2 =mxGetPr(prhs [1 ]);
 	inp3 =mxGetPr(prhs [2 ]);
 	inp4 =mxGetPr(prhs [3 ]);

	/*Get the fifth input, a scalar*/
	leadingDim =mxGetScalar(prhs [4 ]);

	/*Get the dimensions of the first matrix input*/
	mrows =mxGetM(prhs [0 ]);
	ncols =mxGetN(prhs [0 ]);


	//standard case  is leadingDim=1 ==> gives back a row summing columns
	if (leadingDim==1){
		/*Set the output pointer to the output row matrix.*/
		plhs [0 ] = mxCreateDoubleMatrix(1,ncols,mxREAL);
		plhs [1 ] = mxCreateDoubleMatrix(1,ncols,mxREAL);
		plhs [2 ] = mxCreateDoubleMatrix(1,ncols,mxREAL);
		plhs [3 ] = mxCreateDoubleMatrix(1,ncols,mxREAL);
		
		/*Create a C pointer to a copy of the output matrix.*/
		out1 =mxGetPr(plhs [0 ]);
		out2 =mxGetPr(plhs [1 ]);
		out3 =mxGetPr(plhs [2 ]);
		out4 =mxGetPr(plhs [3 ]);
		
		/*Call the C subroutine.*/
		
		int i, j;
		for ( j=0; j<ncols; j++) {
			qd_real cum(0,0,0,0);	
			for (i=0; i<mrows; i++) {
				/* index i+ncols*j ranges from 0 to (mrows-1)+ncols*(ncols-1)*/
				qd_real a(inp1[i+mrows*j], inp2[i+mrows*j],inp3[i+mrows*j], inp4[i+mrows*j]);			
				cum=cum+a;
			}
			out1[j] = cum.x[0];
			out2[j] = cum.x[1];
			out3[j] = cum.x[2];
			out4[j] = cum.x[3];
		}
		
	}
	if (leadingDim==2){
		//gives back a column summing the rows
		/*Set the output pointer to the output column matrix.*/
		plhs [0 ] = mxCreateDoubleMatrix(mrows,1,mxREAL);
		plhs [1 ] = mxCreateDoubleMatrix(mrows,1,mxREAL);
		plhs [2 ] = mxCreateDoubleMatrix(mrows,1,mxREAL);
		plhs [3 ] = mxCreateDoubleMatrix(mrows,1,mxREAL);
		
		/*Create a C pointer to a copy of the output matrix.*/
		out1 =mxGetPr(plhs [0 ]);
		out2 =mxGetPr(plhs [1 ]);
		out3 =mxGetPr(plhs [2 ]);
		out4 =mxGetPr(plhs [3 ]);
		
		/*Call the C subroutine.*/
		
		int i, j;
		for (i=0; i<mrows; i++) {
			qd_real cum(0,0,0,0);	
			for ( j=0; j<ncols; j++) {
				/* index i+ncols*j ranges from 0 to (mrows-1)+ncols*(ncols-1)*/
				qd_real a(inp1[i+mrows*j], inp2[i+mrows*j],inp3[i+mrows*j], inp4[i+mrows*j]);			
				cum=cum+a;
			}
			out1[i] = cum.x[0];
			out2[i] = cum.x[1];
			out3[i] = cum.x[2];
			out4[i] = cum.x[3];
		}		
	}
}

