#include <mex.h>
#include "qd.h"
#include "qd_inline.h"

/*The gateway routine */
void mexFunction(int nlhs,mxArray *plhs [],	int nrhs,const mxArray *prhs [])
{
 	double *inp1, *inp2, *inp3, *inp4;
	int mrows,ncols;
	
	char *input_buf,*output_buf;
	int buflen,status;
	
	/*Check for proper number of arguments.*/
	/*NOTE:You do not need an else statement when using
	mexErrMsgTxt within an if statement.It will never
	get to the else statement if mexErrMsgTxt is executed.
	(mexErrMsgTxt breaks you out of the MEX-file.)
	*/
	if (nrhs !=5)
		mexErrMsgTxt("Five inputs required.");
	if (nlhs !=1)
		mexErrMsgTxt("One output required.");
	
	
	if (!mxIsDouble(prhs [1 ]) || mxIsComplex(prhs [1 ]) || !mxIsDouble(prhs [2 ]) || mxIsComplex(prhs [2 ]) ){
		mexErrMsgTxt("Input matrix type should be ok.");
	}
	
	/*Input must be a string.*/
	if (mxIsChar(prhs [0 ])!=1)
		mexErrMsgTxt("First input must be a string.");
	
	/*Input must be a row vector.*/
	if (mxGetM(prhs [0 ])!=1)
		mexErrMsgTxt("Input string must be a row vector.");
	
	/*Create a pointer to the input matrices.*/
	inp1 =mxGetPr(prhs [1 ]);
	inp2 =mxGetPr(prhs [2 ]);
	inp3 =mxGetPr(prhs [3 ]);
	inp4 =mxGetPr(prhs [4 ]);
	
	/*Get the dimensions of the first matrix input y.*/
	mrows =mxGetM(prhs [1 ]);
	ncols =mxGetN(prhs [1 ]);
	
	/*Get the length of the input string.*/
	buflen =(mxGetM(prhs [0 ])*mxGetN(prhs [0 ]))+1;
	/*Allocate memory for input string.*/
	input_buf = (char*)mxCalloc(buflen,sizeof(char));
	
	status =mxGetString(prhs [0 ] ,input_buf,buflen);
	if (status !=0)
		mexWarnMsgTxt("Not enough space. String is truncated.");
	
	/*Allocate space for the answer; how large? don't know*/
	buflen = 40 * ((mxGetM(prhs [1 ])*mxGetN(prhs [1 ]))+1);
	output_buf = (char*)mxCalloc(buflen,sizeof(char));
	strcpy(output_buf,"");

	char tmp_buf[40];
	qd_real zero(0,0,0,0),two(2,0,0,0),small;
	small=pow(two,-1023);/*This should be changed! It is the value used for dd-type!

	/*Call the C subroutine.*/
	int i,j;
	for (j=0;j<ncols;j++) {
		for (i=0;i<mrows;i++) {
		/*use as index [i+mrows*j]
		    minimum taken value is obviously 0
			maximum value is (mrows-1)+mrows*(ncols-1)=mrows*ncols-1*/
			qd_real a(inp1[i+mrows*j], inp2[i+mrows*j],inp3[i+mrows*j], inp4[i+mrows*j]);
			if (abs(a)<=small) {
			    //put exactly a zero instead of the true value
				zero.write(tmp_buf, 32);			
				strcat(output_buf,tmp_buf);			}
			else{
				a.write(tmp_buf, 32);			
				strcat(output_buf,tmp_buf);
			}
		}
	}
	
	/*Set C-style string output_buf to MATLAB mexFunction output*/
	plhs [0 ] = mxCreateString(output_buf);
	return;
}
