function [R,p] = chol(X)
%chol: Cholesky decomposition

%Implemented based upon choldc(a,n,np,p) from NUMERICAL RECIPES in FORTRAN 
%by CLV, 20041211

%here follows the original FORTRAN code
% INTEGER n,np
% REAL a(np,np),p(n)
% Given a positive-definite symmetric matrix a(1:n,1:n), with physical dimension np this
% routine constructs its Cholesky decomposition,A = L*L' . On input, only the upper triangle
% of a need be given;it is not modifed. The Cholesky factor L is returned in the lower triangle
% of a except for its diagonal elements which are returned in p(1:n).
% INTEGER i,j,k
% REAL sum
% do 13 i=1,n
% 	do 12 j=i,n
% 	  sum=a(i,j)
% 	  do 11 k=i-1,1,-1
% 	    sum=sum-a(i,k)*a(j,k)
% 	  enddo 11
% 	if(i.eq.j)then
% 	  if(sum.le.0.)pause "choldc failed" 
%       p(i)=sqrt(sum)
% 	  else
% 	    a(j,i)=sum/p(i)
% 	  endif
% 	enddo 12
% enddo 13
% return
% END

%now the slow (but workable!) matlab version
n=size(X,1);count=0;
% try
%     X(n,n);%trap the problem with the subsref
% catch
%     disp('problem with the subsref trapped!')
%     hi=gethi(X);lo=getlo(X);
%     X=dd(hi,lo);
% end
%S.type='()';
for i=1:n
    for j=i:n
%         S.subs={i,j};
%         sum=subsref(X,S);
        sum=X(i,j);
        for k=(i-1):-1:1
            sum=sum-X(i,k)*X(j,k);
        end
        if i==j
            if (sum<0) 
                if nargout<2
                    error('X, with rounding errors, is not positive definite')
                else
                    count=[count i];
                end
            end
            p(i)=sqrt(sum);
        else
            X(j,i)=sum/p(i);
        end
    end
end
%the result is stored in the lower triangle part of X; the diagonal terms are stored in p
R=tril(X,-1)'+diag(p);
%matlabs gives back a scalar. Apparently is simple the first row which produces a non-positive
%sum. 
p=max(count);
if p>0
    %Non-positive definite matrix; give back just the already processed part
    p=count(2);%ignore the leading zero
    t=1:(p-1);
    R=R(t,t);
end