function [U,S,V]=mysvd(A,dummy)
%function [U,S,V]=mysvd(A,dummy)
%  SVD    Singular value decomposition.
%     [U,S,V] = SVD(X) produces a diagonal matrix S, of the same 
%     dimension as X and with nonnegative diagonal elements in
%     decreasing order, and unitary matrices U and V so that
%     X = U*S*V'.
%  
%     S = SVD(X) returns a vector containing the singular values.
%  
%     [U,S,V] = SVD(X,0) produces the "economy size"
%     decomposition. If X is m-by-n with m > n, then only the
%     first n columns of U are computed and S is n-by-n.
%svd: singular value decomposition
%Outputs should verify that U*S*V'==A 
%The case with two arguments is considered as an "economy size" request

%Based on SLATEC library; standard call will look as:
%[x,ldx,n,p,s,e,u,ldu,v,ldv,work,job,info]=csvdc(x,ldx,n,p,s,e,u,ldu,v,ldv,work,job,info,varargin);
% where:
%         X         COMPLEX(LDX,P), where LDX .GE. N.
%                   X contains the matrix whose singular value
%                   decomposition is to be computed.  X is
%                   destroyed by CSVDC.
%In our case, x=A;
%
%         LDX       INTEGER.
%                   LDX is the leading dimension of the array X.
%In our case, ldx=size(A,1);
%
%         N         INTEGER.
%                   N is the number of rows of the matrix X.
%In our case, n=ldx;
%
%         P         INTEGER.
%                   P is the number of columns of the matrix X.
%In our case, p=size(A,2);
%
%         LDU       INTEGER.
%                   LDU is the leading dimension of the array U
%                   (see below).
%In our case, ldu=size(A,1);
%
%         LDV       INTEGER.
%                   LDV is the leading dimension of the array V
%                   (see below).
%In our case, ldu=size(A,2);
%
%         WORK      COMPLEX(N).
%                   WORK is a scratch array.
%In our case, work=zeros(size(A,1),1);
%
%         JOB       INTEGER.
%                   JOB controls the computation of the singular
%                   vectors.  It has the decimal expansion AB
%                   with the following meaning
%
%                        A .EQ. 0    Do not compute the left singular
%                                    vectors.
%                        A .EQ. 1    Return the N left singular vectors
%                                    in U.
%                        A .GE. 2    Return the first MIN(N,P)
%                                    left singular vectors in U.
%                        B .EQ. 0    Do not compute the right singular
%                                    vectors.
%                        B .EQ. 1    Return the right singular vectors
%                                    in V.
%In our case: 
if nargin<2
    job=0;
else
    if dummy~=0
        job=0;
    else
        job=11;
    end
end
job=11
%     On Return
%
%         S         COMPLEX(MM), where MM = MIN(N+1,P).
%                   The first MIN(N,P) entries of S contain the
%                   singular values of X arranged in descending
%                   order of magnitude.
%In our case, we preallocate as 
s=zeros(min(size(A,1)+1,size(A,2)));
%
%         E         COMPLEX(P).
%                   E ordinarily contains zeros.  However see the
%                   discussion of INFO for exceptions.
e=zeros(size(A,2),1);
%
%         U         COMPLEX(LDU,K), where LDU .GE. N.  If JOBA .EQ. 1
%                                   then K .EQ. N.  If JOBA .GE. 2 then
%                                   K .EQ. MIN(N,P).
%                   U contains the matrix of right singular vectors.
%                   U is not referenced if JOBA .EQ. 0.  If N .LE. P
%                   or if JOBA .GT. 2, then U may be identified with X
%                   in the subroutine call.
u=pi;
%
%         V         COMPLEX(LDV,P), where LDV .GE. P.
%                   V contains the matrix of right singular vectors.
%                   V is not referenced if JOB .EQ. 0.  If P .LE. N,
%                   then V may be identified with X in the
%                   subroutine call.
v=pi;
%
%         INFO      INTEGER.
%                   The singular values (and their corresponding
%                   singular vectors) S(INFO+1),S(INFO+2),...,S(M)
%                   are correct (here M=MIN(N,P)).  Thus if
%                   INFO.EQ. 0, all the singular values and their
%                   vectors are correct.  In any event, the matrix
%                   B = CTRANS(U)*X*V is the bidiagonal matrix
%                   with the elements of S on its diagonal and the
%                   elements of E on its super-diagonal (CTRANS(U)
%                   is the conjugate-transpose of U).  Thus the
%                   singular values of X and B are the same.
info=pi;
[x,ldx,n,p,s,e,u,ldu,v,ldv,work,job,info]=csvdc(A,size(A,1),size(A,1),size(A,2),s,e,u,size(A,1),v,size(A,2),zeros(size(A,1),1),job,info);
if nargout <=1 
    min(size(A,1)+1,size(A,2))
    U=s(:);%Indeed the singular values as column vector
else
    U=u;S=diag(s);V=v;
end